Import-Module *STIG

<#
 .Synopsis
  Get the list of STIGs to implement.

 .Description
  Get the list of installed STIG modules that can be implemented in DSC.
#>
function Get-STIGs
{
    Get-Module -ListAvailable *STIG | ForEach-Object Name
}

<#
 .Synopsis
  Create a new PowerShell DSC configuration data file.

 .Description
  Create a new PowerShell DSC configuration data file into the PowerShell Data format (psd1).

 .Parameter STIG
  Which STIG to implement.

 .Parameter Destination
  Where to output the PowerShell Data format (psd1) file. Defaults to '.\config.psd1'.

 .Example
  # Create a new PowerShell Data format (psd1) file.
  New-STIGConfig -STIG Win2016STIG -Destination '.\config.psd1'
#>
function New-STIGConfig
{
    param(
        [parameter(Mandatory=$true)]
        [String]
        $STIG,
        [String]
        $Destination = '.\config.psd1'
    )
    $STIGPath = Split-Path (Get-Module -ListAvailable $STIG).Path
    $configPath = Join-Path $STIGPath -ChildPath ($STIG + 'config.psd1')
    Copy-Item $configPath -Destination $Destination
}

<#
 .Synopsis
  Create a new PowerShell DSC configuration.

 .Description
  Create a new PowerShell DSC configuration into the Managed Object Format (MOF).

 .Parameter STIG
  Which STIG to implement.

 .Parameter ConfigurationData
  The data about the desired configuration. Defaults to '.\config.psd1'.

 .Parameter OutputPath
  Where to output the Managed Object Format (MOF) file. Defaults to '.\dsc'.

 .Example
  # Create a new Managed Object Format (MOF) file.
  New-MOF -STIG Win2016STIG -ConfigurationData '.\config.psd1'

 .Example
  # Create a new Managed Object Format (MOF) file in the specified directory.
  New-STIGMOF -STIG Win2016STIG -ConfigurationData '.\config.psd1' -OutputPath 'C:\dsc'
#>
function New-STIGMOF
{
    param(
        [parameter(Mandatory=$true)]
        [String]
        $STIG,
        [String]
        $ConfigurationData = '.\config.psd1',
        [String]
        $OutputPath = '.\dsc'
    )
    $configuration = $STIG + 'Configuration'
    & $configuration -ConfigurationData $ConfigurationData -OutputPath $OutputPath
}

<#
 .Synopsis
  Start the PowerShell DSC configuration.

 .Description
  Start the PowerShell DSC configuration from the desired Managed Object Format (MOF).

 .Parameter Path
  The path to the Managed Object Format (MOF) file to start. Defaults to '.\dsc'.

 .Example
  # Start the Desired State Configuration (DSC).
  Start-DSC -Path '.\dsc'
#>
function Start-STIGDSC
{
    param(
        [String]
        $Path = '.\dsc'
    )
    Start-DscConfiguration -Path $Path -Wait -Force -Verbose
}

function Get-Resource-States
{
	param(
		[parameter(Mandatory=$true)]
		[String]
		$STIG,
		[parameter(Mandatory=$true)]
		[array]
		$Resources,
		[parameter(Mandatory=$true)]
		[boolean]
		$State
	)
	$out = @()
	$Resources | select ResourceId | foreach {
		if ($_ -match '.*_(?<id>\d+)') {
			$id = $Matches.id
			$rev = Select-String -Path $STIG -Pattern "SV-${id}r(\d)_rule" -List |% { $_.matches.groups[1].value }
			if (-not $rev) { $rev = 0 }
			$out += [PSCustomObject]@{
				Name = "xccdf_mil.disa.stig_rule_SV-${id}r${rev}_rule"
				Result = If ($State) {"pass"} Else {"fail"}
			}
		}
	}
	return $out
}

function Write-Xml-Results
{
	param(
		[parameter(Mandatory=$true)]
		[Hashtable]
		$Results,
		[parameter(Mandatory=$true)]
		[String]
		$BaseXMLName,
		[parameter(Mandatory=$true)]
		[String]
		$OutputPath
	)
	$xmlsettings = New-Object System.Xml.XmlWriterSettings
	$xmlsettings.Indent = $true
	$xmlsettings.IndentChars = "    "

	$XmlWriter = [System.XML.XmlWriter]::Create($OutputPath, $xmlsettings)

	$xmlWriter.WriteStartDocument()

	$prefix = "cdf"
	$ns = "http://checklists.nist.gov/xccdf/1.2"
	  
	$xmlWriter.WriteStartElement($prefix, "TestResult", $ns)
		$xmlWriter.WriteAttributeString("id", "xccdf_mil.disa.stig_testresult_scap_mil.disa.stig_comp_" + $BaseXMLName)
		$endTime = Get-Date -Format "yyyy-MM-ddTHH:mm:ss"
		$xmlWriter.WriteAttributeString("end-time", $endTime)
		$xmlWriter.WriteElementString($prefix, "target", $ns, [System.Net.Dns]::GetHostName())
		$passing = 0
		foreach ($key in $Results.keys) {
			$name = $key
			$result = $Results[$key]
			$xmlWriter.WriteStartElement($prefix, "rule-result", $ns)
			$xmlWriter.WriteAttributeString("idref", $name)
				if ($result -eq "pass") { $passing += 1 }
				$xmlWriter.WriteElementString($prefix, "result", $ns, $result)
			$xmlWriter.WriteEndElement()
		}
		$xmlWriter.WriteStartElement($prefix, "score", $ns)
		$xmlWriter.WriteAttributeString("maximum", $Results.count)
		$xmlWriter.WriteAttributeString("system", "urn:xccdf:scoring:flat-unweighted")
		$xmlWriter.WriteString("$passing")
		$xmlWriter.WriteEndElement()
	$xmlWriter.WriteEndElement()

	$xmlWriter.Flush()
	$xmlWriter.Close()
}

function Get-Results
{
	param(
		[parameter(Mandatory=$true)]
		[String]
		$STIG,
		[parameter(Mandatory=$true)]
		[array]
		$Status
	)
	if ($Status.ResourcesInDesiredState)
	{
		$passed += Get-Resource-States -STIG $STIG -Resources $Status.ResourcesInDesiredState -State $true
	}
	if ($Status.ResourcesNotInDesiredState)
	{
		$failed += Get-Resource-States -STIG $STIG -Resources $Status.ResourcesNotInDesiredState -State $false
	}
	$rules = @{}
	foreach ($rule in $failed+$passed)
	{
		$key = $rule.Name
		if ($rules[$key] -ne "pass")
		{
			$rules[$key] = $rule.Result
		} else
		{
			#Write-Host "$key : $($rules[$key]) : $($rule.Result)"
		}
	}
	return $rules
}

<#
 .Synopsis
  Write XCCDF results based on the LCM state.

 .Description
  Write XCCDF results XML based on the state of the local configuration manager.

 .Parameter Path
  The path of the results file to write. Defaults to '.\xccdf-results'.

 .Parameter STIG
  The path to the manual STIG XML that has been applied.

 .Example
  # Write the XCCDF results to myresults.xml.
  Write-XCCDF-Results -STIG '.\U_MS_Windows_Server_2016_STIG_V1R8_Manual-xccdf.xml' -Path '.\myresults.xml'
#>
function Write-XCCDF-Results
{
	param(
		[parameter(Mandatory=$true)]
		[String]
		$STIG,
		[String]
		$Path = '.\xccdf-results.xml'
	)
	$outputpath = $ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($Path)
	$status = Get-DscConfigurationStatus
	$results = Get-Results -Status $status -STIG $STIG
	$baseXMLName = (Get-Item $STIG).Basename
	Write-Xml-Results -Results $results -BaseXMLName $baseXMLName -OutputPath $outputpath
}


Export-ModuleMember -Function 'Get-STIGs', 'New-STIGConfig', 'New-STIGMOF', 'Start-STIGDSC', 'Write-XCCDF-Results'